@@ -1,5 +1,6 @@
 import json
 
+import treq
 import yaml
 
 from twisted.trial.unittest import TestCase
@@ -175,7 +176,7 @@ class TestCollectionHandler(BaseHandlerTestCase):
         self.assertEqual(urlspec.kwargs, {
             "collection_factory": self.collection_factory,
         })
-        self.assertEqual(urlspec.regex.pattern, '/root$')
+        self.assertEqual(urlspec.regex.pattern, '/root/$')
 
     def test_initialize(self):
         handler = self.handler_helper.mk_handler()
@@ -188,20 +189,20 @@ class TestCollectionHandler(BaseHandlerTestCase):
 
     @inlineCallbacks
     def test_get(self):
-        data = yield self.app_helper.get('/root', parser='json_lines')
+        data = yield self.app_helper.get('/root/', parser='json_lines')
         self.assertEqual(data, [{"id": "obj1"}, {"id": "obj2"}])
 
     @inlineCallbacks
     def test_get_usage_error(self):
         self.collection.all = raise_usage_error
-        resp = yield self.app_helper.get('/root')
+        resp = yield self.app_helper.get('/root/')
         yield self.check_error_response(
             resp, 400, "Do not push the red button")
 
     @inlineCallbacks
     def test_get_server_error(self):
         self.collection.all = raise_dummy_error
-        resp = yield self.app_helper.get('/root')
+        resp = yield self.app_helper.get('/root/')
         yield self.check_error_response(
             resp, 500, "Failed to retrieve objects.")
         [f] = self.flushLoggedErrors(DummyError)
@@ -210,7 +211,7 @@ class TestCollectionHandler(BaseHandlerTestCase):
     @inlineCallbacks
     def test_post(self):
         data = yield self.app_helper.post(
-            '/root', data=json.dumps({"foo": "bar"}), parser='json')
+            '/root/', data=json.dumps({"foo": "bar"}), parser='json')
         object_id = data["id"]
         self.assertEqual(data, {"foo": "bar", "id": object_id})
         self.assertEqual(
@@ -221,7 +222,7 @@ class TestCollectionHandler(BaseHandlerTestCase):
     def test_post_usage_error(self):
         self.collection.create = raise_usage_error
         resp = yield self.app_helper.post(
-            '/root', data=json.dumps({"foo": "bar"}))
+            '/root/', data=json.dumps({"foo": "bar"}))
         yield self.check_error_response(
             resp, 400, "Do not push the red button")
 
@@ -229,7 +230,7 @@ class TestCollectionHandler(BaseHandlerTestCase):
     def test_post_server_error(self):
         self.collection.create = raise_dummy_error
         resp = yield self.app_helper.post(
-            '/root', data=json.dumps({"foo": "bar"}))
+            '/root/', data=json.dumps({"foo": "bar"}))
         yield self.check_error_response(
             resp, 500, "Failed to create object.")
         [f] = self.flushLoggedErrors(DummyError)
@@ -435,7 +436,7 @@ class TestApiApplication(TestCase):
         app_helper = self.get_app_helper(
             collections=(('/:owner_id/store', collection_factory),))
         result = yield app_helper.request(
-            'GET', '/foo/store', headers={"X-Owner-ID": "owner-1"})
+            'GET', '/foo/store/', headers={"X-Owner-ID": "owner-1"})
         content = yield result.content()
         self.assertEqual(json.loads(content), collection_data['foo'])
 
@@ -446,7 +447,7 @@ class TestApiApplication(TestCase):
         app_helper = self.get_app_helper(
             collections=(('/:owner_id/store', collection_factory),),
             preprocessor=None)
-        result = yield app_helper.request('GET', '/foo/store')
+        result = yield app_helper.request('GET', '/foo/store/')
         content = yield result.content()
         self.assertEqual(json.loads(content), collection_data['foo'])
 
@@ -457,7 +458,7 @@ class TestApiApplication(TestCase):
         app_helper = self.get_app_helper(
             collections=(('/:owner_id/store', collection_factory),),
             preprocessor=lambda handler: succeed("owner-1"))
-        result = yield app_helper.request('GET', '/foo/store')
+        result = yield app_helper.request('GET', '/foo/store/')
         content = yield result.content()
         self.assertEqual(json.loads(content), collection_data['foo'])
 
@@ -469,7 +470,7 @@ class TestApiApplication(TestCase):
         app_helper = self.get_app_helper(
             collections=(('/:owner_id/store', collection_factory),),
             preprocessor=owner_from_oauth2_bouncer(auth_server.url))
-        result = yield app_helper.request('GET', '/foo/store')
+        result = yield app_helper.request('GET', '/foo/store/')
         content = yield result.content()
         self.assertEqual(json.loads(content), collection_data['foo'])
 
@@ -489,7 +490,7 @@ class TestApiApplication(TestCase):
         [_health_route, collection_route, elem_route] = routes
         self.assertEqual(collection_route.handler_class, CollectionHandler)
         self.assertEqual(collection_route.regex.pattern,
-                         "/(?P<owner_id>[^/]*)/store$")
+                         "/(?P<owner_id>[^/]*)/store/$")
         self.assertEqual(collection_route.kwargs, {
             "collection_factory": collection_factory,
         })
@@ -647,7 +648,7 @@ class TestApiApplication(TestCase):
         # We should have the prefix on the front of our other URL paths.
         self.assertEqual(
             collection.regex.pattern,
-            r'/foo/bar/(?P<owner_id>[^/]*)/store$')
+            r'/foo/bar/(?P<owner_id>[^/]*)/store/$')
         self.assertEqual(
             element.regex.pattern,
             r'/foo/bar/(?P<owner_id>[^/]*)/store/(?P<elem_id>[^/]*)$')
@@ -662,7 +663,7 @@ class TestApiApplication(TestCase):
         self.assertEqual(health.regex.pattern, r'/health/$')
         self.assertEqual(
             collection.regex.pattern,
-            r'/(?P<owner_id>[^/]*)/store$')
+            r'/(?P<owner_id>[^/]*)/store/$')
         self.assertEqual(
             element.regex.pattern,
             r'/(?P<owner_id>[^/]*)/store/(?P<elem_id>[^/]*)$')
@@ -677,7 +678,7 @@ class TestApiApplication(TestCase):
 
         # The collection handler should use default logging behaviour.
         handler_logs[:] = []  # Clear logs.
-        yield app_helper.get('/foo/store')
+        yield app_helper.get('/foo/store/')
         self.assertEqual(len(handler_logs), 1)
 
         # The health check handler should suppress logging.
@@ -706,6 +707,9 @@ class TestAuthHandlers(TestCase):
     @inlineCallbacks
     def start_fake_auth_server(self, owner_id=None, code=200):
         def auth_request(request):
+            if request.method != "GET":
+                request.setResponseCode(405)
+                return ""
             request.setResponseCode(code)
             if owner_id is not None:
                 request.setHeader("X-Owner-Id", owner_id)
@@ -722,6 +726,22 @@ class TestAuthHandlers(TestCase):
         """
         raise Exception("This collection_factory should never be called")
 
+    @inlineCallbacks
+    def test_fake_auth_server_rejects_post(self):
+        """
+        Test that the fake auth server rejects non-GET requests.
+
+        This is important for :meth:`test_owner_from_bouncer_post` below which
+        should fail if the bouncer preprocesser makes a non-POST request.
+        """
+        auth_server = yield self.start_fake_auth_server()
+        # Accepts GET
+        resp = yield treq.get(auth_server.url, persistent=False)
+        self.assertEqual(resp.code, 200)
+        # Rejects POST
+        resp = yield treq.post(auth_server.url, persistent=False)
+        self.assertEqual(resp.code, 405)
+
     def test_owner_from_header_with_value(self):
         preprocessor = owner_from_header("X-Owner-Id")
         handler = self.dummy_helper.mk_handler(
@@ -757,6 +777,16 @@ class TestAuthHandlers(TestCase):
         owner_id = yield preprocessor(handler)
         self.assertEqual(owner_id, "owner-1")
 
+    @inlineCallbacks
+    def test_owner_from_bouncer_post(self):
+        auth_server = yield self.start_fake_auth_server("owner-1")
+        preprocessor = owner_from_oauth2_bouncer(auth_server.url)
+        handler = self.dummy_helper.mk_handler(
+            headers={"Authorization": "Bearer foo"})
+        handler.request.method = "POST"
+        owner_id = yield preprocessor(handler)
+        self.assertEqual(owner_id, "owner-1")
+
     @inlineCallbacks
     def test_owner_from_bouncer_without_value(self):
         auth_server = yield self.start_fake_auth_server(code=401)
